// Persistence of Vision Ray Tracer Scene Description File
// File: Radiosity.pov
// Vers: 3.5
// Desc: Example-File for experiments with Radiosity
// Date: 12.07.03 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak (www.NoLights.de)
// Last Update: 18.07.08 (dd.mm.yy)

//Description
// Scene Stage for experiments using different combinations of
// conventional lighting, radiosity and photons. These are
// all indoor experiments, outdoor may behave somewhat different.

// The file is set to produce the Radiosity- and Photon-Data, but
// then needs to be set to the second pass using the Which_Pass-
// Variable (just set it to 2 instead of 1). You also need to
// set Save_Pho to 0 and Load_Pho to 1 for POV-Ray to reload
// the saved data instead of calculating it again.

// It will then reload all the data saved on the first pass and 
// smoothen the radiosity as described in the Making Of on my
// website (where you got this Source-File from, I hope!).

//Warning
// Radiosity-Scenes may take really long rendering times. This
// file ships with most complicated setup done in the experiments:
// All photons and radiosity calculations done in first step,
// with raised max_trace_level.

//Example_Setup
// Switch for lightsources, settings, objects etc. Note that it does
// NOT set photon-activity, as photons always require a lightsource
// when shot. It is advised to render the scene once with photons,
// and then conduct the tests by loading the photons by hand.
#declare Example_Setup=6;
//1 - Radiosity calculated without photons and without lightsource
//2 - Radiosity done using loaded photons, no lightsource
//3 - Radiosity done using lightsource, no photons
// From here on, brightness is reduced to .5 instead of raised to 1.25
//4 - Radiosity done using lightsource + no_shadow, no photons
//5 - Radiosity done using lightsource + no_shadow + photons
//6 - Radiosity, like experiment 5, but max_trace_level set to 25 and smaller
//    error_bound on 2nd pass

//Which_Pass
// Switch to determine pass-specific options
#declare Which_Pass = 1;
//1 - First pass, blotchy results
//2 - Second pass, smoothened results

//A lightsource is used when saving photons, its
// switched off otherwise. It is advised to render the scene
// once and save photons, and later load that data when
// needed.
#declare Photons=1;
#declare Save_Pho=1;
#declare Load_Pho=0;

//Since this is a Radiosity scene, Radiosity is simply switched on
// by default. When doing the first scene render in order to just save
// the photons, you might want to switch it off.
#declare Radiosity=1;
 #if (Which_Pass!=2)
  #declare Save_Rad=1;
  #declare Load_Rad=0;
 #else
  #declare Save_Rad=0;
  #declare Load_Rad=1;
 #end

//Sampling Radiosity in one pass yields different results than when
// using several passes
#declare One_Pass=false;

global_settings{
 assumed_gamma 1
 #if (Example_Setup=6)
  max_trace_level 25 #else
  max_trace_level 5  #end 
 #if (Radiosity)
 radiosity{
  #if (Save_Rad)
   #if (One_Pass)
    pretrace_start 1
    pretrace_end 1
   #else
    pretrace_start .1
    pretrace_end .005
   #end
   error_bound .1
   save_file "radio.rad"
  #end
  #if (Load_Rad)
   pretrace_start 1
   pretrace_end 1
   #if (Example_Setup<=5)
    error_bound .16
   #else
    error_bound .8
   #end
   load_file "radio.rad"
  #end  
  //Different brightness for 1st and 2nd pass, to compensate
  // for the higher interpolation on the 2nd pass
  #if (Save_Rad)
   brightness 1.00
  #else
   #if (Example_Setup>=4)
    brightness .5
   #else
    brightness 1.25
   #end
  #end
  gray_threshold 0
  low_error_factor 1
  minimum_reuse .0025
  nearest_count 20
  count 1200
  recursion_limit 2
  always_sample off
  }
 #end
 #if (Photons)
 photons{
  spacing .005
  autostop .5
  #if (Save_Pho) save_file "radio.pho" #end
  #if (Load_Pho) load_file "radio.pho" #end
  }
 #end
 }

camera{
 location <0,5,-17>
 //Provides proper Aspect-Ratio
 up y right x*image_width/image_height
 look_at <0,5,0>
 }

#default{finish{ambient 0 diffuse 1}}

//Cornell
box{<-5,-1,-5><-6,10,6> pigment{rgb x}}
box{< 5,-1,-5>< 6,10,6> pigment{rgb y}}
box{<-6,-1, 5>< 6,10,6> pigment{rgb 1}}
box{<-6, 0,-5><6,-1,5> pigment{rgb 1}}
box{<-6,10,-5><6,11,5> pigment{rgb 1}}

box{<-6,-1,-5>< 6,10,-6> pigment{rgb 1} no_image}
//Lightsource
box{<-1,9.25,-1><1,10,1> pigment{rgb 1} finish{ambient #if (Example_Setup>=3) 1 #else 25 #end}
 #if (Example_Setup>=4)
 no_shadow
 #end
 }

#if (Save_Pho | Example_Setup>=3)
light_source{<0,9,0> rgb 1
 //Fading lightsource is important for realistic lighting!
 fade_power 2 fade_distance 6
 //And an arealight for realistic and soft shadows
 area_light x*2,z*2,16,16 orient circular
 photons{reflection on refraction on}
 }           
#end

torus{ 2,.5
 pigment{rgb <.25,.25,1>}
 rotate <-35,-37,0>
 translate <-2,7,2>
 finish{specular 10 roughness .02}
 }

sphere{0,1 scale <1,2,1> translate <-3,2,-1>
 pigment{rgbt <1,1,1,1>}
 finish{reflection{0,1 falloff 2} conserve_energy}
 interior{ior 1.66 fade_power 2 fade_distance 8}
 photons{target reflection on refraction on collect off}
 }

triangle{
 <1,0,-3><4,4,-4><4,1,-2.5>
  pigment{rgb 1}
  finish{reflection 1 conserve_energy}
  photons{target reflection on refraction off collect off}
  }
